/////////////////////////////////////////////////////////////////////////////
//
// File: bin_add.cpp
//
//
// Working example of binary add (fast?-hardly)
// --------------------------------------------
// Uses the scanner to allow easy entry of numbers
// syntax is: <const> "+" <const> | <const> "-" <const>

#include "r1emu.h"
#include <iostream>

bool carry;
bool sign;
bool overflow;
bool zero;


WORD alu_add( WORD a, WORD b, bool sub, bool cin ) {
  // two's complement when subtracting
  if (sub) b = ~b;
  // carry is inverted if sub to become borrow
  cin = cin ^ sub;
	
	// calculate propagate/generate for all
  WORD propagate = a | b;
  WORD generate = a & b;
  WORD res = a + b + (cin?1:0);

  // correct for carry-in (generate[0] = (a[0] & b[0]) | ( cin & (a[0] | b[0]) )
  if (cin && (propagate & 1)) generate |= 1;

  // ASSUME LEFT-SHIFT INSERTS 0!
  bool msbprop = (propagate & 0x80000000); propagate <<= 1;
  bool msbgen = (generate & 0x80000000); generate <<= 1;
  bool msbcarry;
  while ( (propagate & 0x80000000) && !(generate & 0x80000000) ) {
    propagate <<= 1;
    generate <<= 1;
	}
  msbcarry = generate & 0x80000000;
  carry = msbgen || (msbprop && msbcarry);
  overflow = carry ^ msbcarry;
  sign = (res & 0x80000000) > 0;
  zero = res == 0;
  return res;
}
	


int main() {
        WORD a, b;
        char c;
        cout << "\nEnter <number> -/+ <number>: ";
        cin >> a >> c >> b;
//        a = (WORD)-2000000000; b = 1000000000; c = '-';
	WORD r = alu_add(a,b, c=='-', true);
	cout << "a=" << (INT)a << " b=" << (INT)b << " op= " << c << endl;
        cout << "c=" << carry << " s=" << sign << " of=" << overflow << "  a+b=" << (INT)r << endl;
}



