/////////////////////////////////////////////////////////////////////////////
//
// File: console.h
// ---------------
// Interface to console services
// Defines:
//     - Console : interface to low-level console services
//     - ConEdit : higher level interface for editing fields

#include "console.h"
#include <conio.h>
#include <stdio.h>
#include <stdlib.h>
#include <pc.h>
#include <dos.h>

// try to instantiate a console session
Console::Console() {
	textmode(C4350);
}

Console::~Console() {
	normcursor();
	textmode(C80);
}

// move cursor to position on screen
void Console::gotoxy(int x, int y) {
	::gotoxy(x,y);
}

// clear screen
void Console::clrscr() {
	::clrscr();
}

//
void Console::clreol() {
	::clreol();
}
		
// set the text attributes on the console
void Console::textattr( int ta) {
	curattr = ta;
	::textattr(curattr);
}

void Console::textcolor( int color ) {
	::textcolor(color);
}

void Console::textbackground( int background ) {
	::textbackground(background);
}


// overloaded versions of print for 
void Console::printstring( string str) {
	cprintf("%s", str.c_str());
}

void Console::print( char* cstr) {
	cprintf("%s", cstr);
}

void Console::print( char c ) {
	putch(c);
}
	
		
// return columns on screen
int Console::cols() {
	return 80;
}
		
// return number of rows on screen
int Console::rows() {
	return 50;
}
		
// attempt n lines, if success, returns true, otherwise false
bool Console::setRowCount( int r ) {
	//if (r==50) setTextMode(C4350);
	return true;
}
		
// attempt n cols, if success, returns true, otherwise false
bool Console::setColCount( int c ) {
	return false;
}
	

int Console::wherex() {
	return ::wherex();
}

int Console::wherey() {
	return ::wherey();
}

int Console::readkey() {
	return ::getch();
}

bool Console::keyhit() {
	return ::kbhit();
}

void Console::blockcursor() {
	::_setcursortype(_SOLIDCURSOR);
}

void Console::normcursor() {
	::_setcursortype(_NORMALCURSOR);
}


///////////////////////////////////////////////////////////////////////////////

// clears memory
void memclr( char *pnt, long int size ) {
	int lp1=0;
	for ( ; lp1<size; lp1++, pnt++)
		*pnt = 0;
}


// deletes character at index pos
void strdel( char *str, int pos )
{
	int lp1;
	int len = strlen(str);

	if (pos>=len) return;

	str +=pos;
	for (lp1=pos; lp1<len; lp1++, str++)
		*str = *(str+1);
	*(str+1) = 0;
}


// inserts character in string at position pos (all chars moved)
// assumes buffer is large enough
void strins( char *str, int pos, char ch )
{
	char *s = str;
	int lp1;
	int len = strlen(str);
	if (pos>len) return;

	if (pos==len) {
		*(str+len) = ch;
		*(str+len+1) = 0;
		return;
	}

	s+=len;
	for (lp1=len; lp1>=pos; lp1--, s--)
		*(s+1) = *s;
	*(s+1) = ch;
}

// Definition of bios keycodes
#define KC_LEFT  75
#define KC_RIGHT 77
#define KC_UP    72
#define KC_DOWN  80
#define KC_INS   82
#define KC_HOME  71
#define KC_END   79
#define KC_DEL   83
#define KC_ESC   27
#define KC_SHIFT_TAB 15
#define KC_TAB   '\t'
#define KC_BACKSPACE '\b'
#define KC_RETURN '\r'
#define KC_ENTER '\r'


// Create a new editing field
ConEdit::ConEdit(Console* con, int x, int y, bool ro, int maxlen, int col=0x70, char* defval=NULL):
		console(con),
		xPos(x), yPos(y), readOnly(ro),
		maxLen(maxlen),
		inputAttr(col), inputLen(0), inputPos(0) {
	input = new char[maxlen+1];
	memclr( input, maxLen+1 );
	if (defval) 
		strcpy( input, defval );
}


#define MIN(a,b) ((a)>(b)?(b):(a))

// Handle a field
enum KeyPress ConEdit::process() {
	static bool insert=true;
	enum KeyPress retval = kpNone;
	bool redraw;      // Does the field need redrawing
	unsigned char ch;
	int lp1 = 0;
	// set input-etc to normal
	inputLen = strlen(input);
	inputPos = MIN(inputPos, inputLen);

	// Write field to console
	console->gotoxy(xPos, yPos);
	console->textattr(inputAttr);
	for (lp1=0; lp1<maxLen; lp1++)
		console->print(lp1>inputLen?' ':input[lp1]);
	console->gotoxy(xPos+inputPos, yPos);

	// Enter keyprocessing 
	if (readOnly) retval = kpNext;
	while (retval == kpNone) {

		ch = console->readkey();

		redraw = false;

		// process extended keys (first char is 0)
		if (ch==0) switch (ch=console->readkey()) {

			case KC_LEFT:
				if (inputPos>0) inputPos--;
				else errorcode(1);
				break;

			case KC_RIGHT:
				if (inputPos+1<=inputLen) inputPos++;
				else errorcode(2);
				break;

			case KC_UP: case KC_SHIFT_TAB:
				validate();
				retval = kpPrev;
				break;

			case KC_DOWN:
				validate();
				retval = kpNext;
				break;

			case KC_HOME:
				inputPos=0;
				break;

			case KC_END:
				inputPos = inputLen;
				break;

			case KC_INS:
				insert = !insert;
				if (!insert) console->blockcursor(); else console->normcursor();
				break;

			case KC_DEL:
				if (inputLen>0 && inputPos<inputLen) {
					strdel(input, inputPos);
					inputLen--;
					redraw = true;
				} else
					errorcode(3);
				break;

		} else

		// Ordinary edit key
		if (ch<32) switch (ch) {
			case KC_BACKSPACE:
				if (inputLen>0 && inputPos>0) {
					inputPos--;
					inputLen--;
					strdel(input, inputPos);
					redraw = true;
				} else
					errorcode(3);
				break;

			case KC_RETURN:
				validate();
				retval = kpDone;
				break;

			case KC_TAB:
				validate();
				retval = kpNext;
				break;

			case KC_ESC:
				validate();
				retval = kpEsc;
				break;

		} else

		// entry keys
			if (inputLen<maxLen) {
				if (insert) {
					strins(input,inputPos,ch);
					inputPos++;
					inputLen++;
				} else {
					if (inputPos==inputLen) {
						strins(input, inputPos, ch);
						inputPos++;
						inputLen++;
					} else {
						input[inputPos] = ch;
						inputPos++;
					}
				}
				redraw = true;
			} else
				errorcode(4);


		// if the field has changed, do a complete redraw
		if (redraw || retval!=kpNone) {
			console->textattr(inputAttr);
			console->gotoxy(xPos, yPos);
			for (lp1=0; lp1<maxLen; lp1++)
				console->print(lp1>inputLen?' ':input[lp1]);
		}
		console->gotoxy(xPos+inputPos, yPos);

	} // while

	return retval;
}


void ConEdit::errorcode(int ec) {
	console->gotoxy(1,50); 
	console->clreol();
	switch (ec) {
		case 1: 
			console->print("EDIT001: Cannot move cursor left");
			break;
		case 2: 
			console->print("EDIT002: Cannot move cursor right");
			break;
		case 3: 
			console->print("EDIT003: Cannot delete character here");
			break;
		case 4: 
			console->print("EDIT004: Max length reached, nothing more to enter");
			break;
		default:;
	}
	while (!console->keyhit()) ;
	console->gotoxy(1,50); console->clreol();
}
		
void ConEdit::setText( char* newtext ) {
	char *src = newtext;
	char *dest = input;
	int cnt = 0;
	// copy data from newtext to input
	while (cnt<maxLen) {
		*dest = *src;
		if (*src) src++;
		dest++;
		cnt++;
	}
}

void ConEdit::getText( char* dest ) {
	char *src = input;
	while (*src) {
		*dest = *src;
		src++; dest++;
	}
	*dest=0;
}

int ConEdit::getTextLen() {
	return inputLen;
}