#ifndef CONSOLE_H
#define CONSOLE_H
/////////////////////////////////////////////////////////////////////////////
//
// File: console.h
// ---------------
// Interface to console services

#include <string>

class Console {
	public:
		// try to instantiate a console session
		Console();
		~Console();
		
		// move cursor to position on screen
		void gotoxy(int, int);
		
		// clear screen
		void clrscr();
		
		// clear 'till end of line
		void clreol();
		
		// set the text attributes on the console
		// format is ((backcolor << 4) | frontcolor)
		void textattr( int );
		void textcolor( int );
		void textbackground( int );
		
		// versions of print for 
		void printstring( string );
		void print( char* );
		void print( char );
		
		// return columns on screen
		int cols();
		
		// return number of rows on screen
		int rows();
		
		// cursor position
		int wherex();
		int wherey();
		// store current xy-position (for interrupted writes)
		//void pushxy(); 
		
		// read one character from the console
		int readkey();
		// look for keypresses
		bool keyhit();
		
		// attempt n lines, if success, returns true, otherwise false
		bool setRowCount( int );
		
		// attempt n cols, if success, returns true, otherwise false
		bool setColCount( int );
		
		void blockcursor();
		void normcursor();
		
		enum {
			/*  dark colors     */
			BLACK, BLUE, GREEN, CYAN, RED, MAGENTA, BROWN, LIGHTGRAY,
			/*  light colors    */
			DARKGRAY, LIGHTBLUE, LIGHTGREEN, LIGHTCYAN, LIGHTRED, LIGHTMAGENTA, YELLOW, WHITE };
	protected:
		int curattr;

};

//
typedef unsigned int uint;
// A field may return the following results during editing
enum KeyPress { kpNone, kpDone,kpNext, kpPrev, kpEsc };

// ConEdit defines an editing entry field on screen
// ConEdit handles all the tiresome details of editing keys etc.
class ConEdit {
	protected:
		uint tag;			// tag may be used freely
		void *context;		// context may be used freely
		uint xPos;			// x-position on console
		uint yPos;			// y-position on console
		bool readOnly;		// mark this editing field as read-only
		uint maxLen;		// Max length of entryfield
		uint inputPos;		// current editing position in field
		uint inputLen;    // Length of input in field
		int  inputAttr;   // Textattributes for input field
		char *input;      // Buffer to store input in
		Console* console;	// The console on which drawing takes place...
	public:
		// construct a new edit field
		ConEdit(Console* con, int x, int y, bool ro, int maxlen, int col=0x70, char* defval=NULL);
	
		// Process this edit field
		enum KeyPress process();
		// validate the contents of this field
		virtual bool validate() { return true; }
		// Process may call errorcode with:
		//  1 - cursor can't move left
		//  2 - cursor can't move right
		//  3 - nothing to delete
		//  4 - max length reached
		virtual void errorcode(int);
		
		// returns the length of the input in the field. Alloc getTextLen()+1 for getText()
		int  getTextLen();
		// read the text from the field and put it into the buffer
		void getText( char* );
		// set the input to 
		void setText( char* );
		
};

#endif
