#ifndef CPU_H
#define CPU_H
/////////////////////////////////////////////////////////////////////////////
//
// File: cpu.h
//
//
// The R1-emulator
// ---------------
//
// CPU defines the class implementing all the functionality
// of the CPU, like memory, interrupts eg.
//
//

#include <string>
#include "r1emu.h"
#include "timer.h"
#include "interrupt.h"

#define R_IP regs[0]
#define R_FL regs[1]
#define R_FLAGS (*((struct Flags*)&regs[1]))
#define R_SP regs[2]
#define R_BP regs[3]
#define R_IT regs[4]
#define BOOT_UP_VECTOR 0L


class CPU {
	public:
 		// Construct with memory-size
 		CPU( WORD memsize );
		// Construct default mem-size, load rom-image from fname
		CPU( string fname );
		// Destructor - release memory-areas etc.
		~CPU();


		// Step one instruction 
		int step();
		
		// Run whatever is in memory - returns when CPU is halted.
		void run();

		// Current cycle-count
		unsigned long cycleCount;
		
		// An interrupt priority decoder
		IRQDecoder *irq;

		// The four timers
		Timer *timer0, *timer1, *timer2, *timer3;
		
		// name of image loaded
		string imageFile;

		// array for registers
		WORD regs[32];
		INT  *iregs;

		// Memory storage for this CPU
		BYTE *memory;
		WORD mem_size;

		// Memory Access functions
		inline WORD	memw( WORD address ); // Read a word from memory
		inline BYTE	memb( WORD address ); // Read a half-word from memory
		inline void	wmemw( WORD address, WORD value ); // Write word to memory
		inline void	wmemb( WORD address, BYTE value ); // Write half-word to memory

		// Method to emulate ALU (update flags on add/sub!)
		WORD inline alu_add( WORD a, WORD b, bool sub, bool cin );

		// state of CPU (HALTED or RUNNING)
		enum { RUNNING, HALTED } state;

}; // end class definition

#endif
