/////////////////////////////////////////////////////////////////////////////
//
// File: interrupt.cpp
// -----------------
// 

#include "interrupt.h"

// constructor resets memory areas used by the IRQDecoder
IRQDecoder::IRQDecoder() {
	for (int lp1=0; lp1<32; lp1++) {
		servicing[lp1] = false;
		irq[lp1] = false;
	}
}

// call request to request an interrupt of type no
void IRQDecoder::request( int no ) {
	if (no>=0 && no<=31) {
		irq[no] = true;
	}
}

	
// call start(_) when you start handling interrupt number "no"
void IRQDecoder::start( int no ) {
	if (no>=0 && no<=31 && irq[no]) {
		servicing[no] = true;
	}
}

	
// call complete when done servicing currently highest priority int. being handled
void IRQDecoder::done() {
	for (int no=0; no<32; no++)
		if (no>=0 && no<=31 && irq[no] && servicing[no] ) {
			servicing[no] = false;
			irq[no] = false;
			return;
		}
}


// call find() to find the next interrupt to service. If you choose to 
// service that interrupt, call start(_). If no new interrupt is found,
// -1 is returned, meaning that no higher-priority interrupts are waiting
// to be serviced.
int IRQDecoder::find() {
	for (int lp1=0; lp1<32; lp1++) {
		if (servicing[lp1]) 
			break;
		if (irq[lp1] && !servicing[lp1]) {
			return lp1;
		}
	}	
	return -1;
}


// Look if interrupt is requested
bool IRQDecoder::isReq( int no ) {
	return irq[no];
}



void IRQDecoder::state(ostream& out) {
	out << "IRQ[";
	for (int lp1=0; lp1<32; lp1++) {
		if (servicing[lp1]) out << "H"; else
		if (irq[lp1]) out << "r"; else
		out << "-";
	}
	out << "]";
}


