#ifndef SCANNER_H
#define SCANNER_H

#include <string>
#include <map>
#include "r1emu.h"

enum ScTokens {
	TT_EOL,			// end-of-line
	TT_ID,			// identifier
	TT_CONST,		// number
	TT_STRING,		// string
	// Paranthesis and brackets '[' ']' '(' ')'
	TT_LB, TT_RB, TT_LP, TT_RP,
	// Operators: ',' '.' '=' ':' '@' '+' '-' '*'
	TT_COMMA, TT_DOT, TT_EQ, TT_COLON,
	TT_AT, TT_PLUS, TT_MINUS, TT_MULT,
	// Instructions
	TT_NOP, TT_ADD, TT_ADC, TT_SUB, TT_SBB, TT_CMP, TT_AND, TT_TEST, TT_OR, TT_XOR,
	TT_BTT, TT_BTR, TT_BTC, TT_BTS, TT_SHR, TT_SAR, TT_SHL, TT_ROR, TT_RCR, TT_ROL, TT_RCL,
	TT_MUL, TT_IMUL, TT_DIV, TT_IDIV,
	TT_MOVE, 
	TT_CLR, TT_SET, TT_NOT, TT_NEG, TT_INC, TT_DEC, TT_PUSH, TT_POP,
	TT_JMP, TT_CALL, TT_JE, TT_JNE, TT_JC, TT_JNC, TT_JS, TT_JNS,
	TT_JA, TT_JAE, TT_JB, TT_JBE, TT_JG, TT_JGE, TT_JL, TT_JLE,
	TT_RET, TT_IRET, TT_HALT,
	// registers - R00..R31, IP, BP, SP, IT, FLAGS
	TT_REG,
	// storage-specifiers
	TT_DB, TT_DW,
	// keyword
	TT_SEG, TT_END, TT_ASSUME
};


class Scanner {
	public:
		Scanner(bool = true);
		~Scanner();

		// Initialise scanner for new line.
		// All internal fields are reset
		void newLine( string );

		// Return the current lexeme
		string getLex();

		// Get a value from the input (if current is val)
		// May throw an exception if incorrect token
		WORD getWord();

		// Read the next token from the line
		// The type of the token is returned
		ScTokens next();

		// Returns the current token type.
		ScTokens curTok() { return tok; }

 protected:
		char line[256];
		int linepos;
		WORD val;
		string lex;
		ScTokens tok;
		bool chk_kw;

		void scId();
		void scVal();
		void scString();
		void check_keyword();

};

#endif

